!
! Version 30.04.2002 - H. Forbert   harald.forbert@theochem.ruhr-uni-bochum.de
!
MODULE read_data 

  USE kinds, ONLY : dbl, filename_type

  IMPLICIT NONE

  PRIVATE
  PUBLIC :: check_file, read_file

CONTAINS

!! counts number of lines in file 'filename' to read
!! the format is:
!! i a2 a3 a4 a5 a6 a7
!! i : some integer index
!! a2,a3,a4 : real's that get ignored
!! a5,a6,a7 : 3 reals - vector of data

SUBROUTINE check_file(filename, length, dim)

  IMPLICIT NONE

  TYPE ( filename_type ), INTENT(IN) :: filename
  INTEGER, INTENT(OUT) :: length
  INTEGER, INTENT(OUT) :: dim

  INTEGER :: n, ios
  INTEGER :: i
  REAL (dbl) :: a2,a3,a4,a5,a6,a7

  OPEN (UNIT = 10, FILE = filename%name, STATUS = "OLD", ACTION = "READ", &
    IOSTAT = ios )
  IF (ios /= 0) THEN
     PRINT *,"Error opening file ",filename%name
     STOP
  END IF

  n = 0
  DO 
     READ (10,*,IOSTAT=ios) i,a2,a3,a4,a5,a6,a7
     IF (ios /= 0) EXIT
     n = n + 1
  END DO

  CLOSE(UNIT=10, IOSTAT=ios)

  IF (ios /= 0) THEN
     PRINT *,"Error closing file ", filename%name
     STOP
  END IF

  length = n
  dim = 1

END SUBROUTINE check_file

!! read a file in the same format as 'check_file'
!! input array will hold the data. The array must be
!! dimensioned sufficiently large in each dimension.

SUBROUTINE read_file(input, filename, nn, dd)

  IMPLICIT NONE

  REAL(dbl), DIMENSION ( : , : ), INTENT(INOUT)  :: input
  TYPE ( filename_type ), INTENT(IN) :: filename
  INTEGER, OPTIONAL :: nn
  INTEGER, OPTIONAL :: dd

  INTEGER :: ix, j, i, n, d, ios
  REAL (dbl) :: a2,a3,a4

  OPEN (UNIT = 10, FILE = filename%name, STATUS = "OLD", ACTION = "READ", &
    IOSTAT = ios )
  IF (ios /= 0) THEN
     PRINT *,"Error opening file ",filename%name
     STOP
  END IF

  n = SIZE(input,1)
  d = SIZE(input,2)
  IF (PRESENT(nn) .AND. (nn < n)) n = nn
  IF (PRESENT(dd) .AND. (dd < d)) d = dd

  DO i = 1, n
    DO j = 0, d-3, 3
      READ (10,*,IOSTAT=ios) ix,a2,a3,a4,input(i,1+j),input(i,2+j),input(i,3+j)
      IF (ios /= 0) THEN
         PRINT *,"Error reading file ",filename%name
         STOP
      END IF
    END DO
  END DO

  CLOSE(UNIT=10, IOSTAT=ios)

  IF (ios /= 0) THEN
     PRINT *,"Error closing file ", filename%name
     STOP
  END IF

  RETURN

END SUBROUTINE read_file

END MODULE read_data
